#!/bin/sh 

# Script to start the various needed VisiBroker servers for Gee
#
# usage: vbstart [-q] [port-number]
#
#	where -q turns off the verbose flags to the servers
#		(the default is to invoke the servers in verbose mode)
#	and where port-number is the number for the OSAGENT_PORT
#
# You should avoid using port-number.  Instead, you should have
# set OSAGENT_PORT set in your environment.
# Each developer should have a separate value for this variable
# to avoid interference between tests.

# Logs of the transactions are maintained in /tmp/${LOGNAME}/
# Also in that directory is a file `pids' which contains the
# process ids of processes to be killed to stop these servers.
# The processes are killed by `vbstop`

vbroker=GEE_VISIBROKER
logpath=/tmp/${LOGNAME}

# set verbosity
if [ "$1" = "-q" ] ; then
  verbflag=
  verbloc=
  shift
else
  verbflag=-v
  verbloc="-LOCdebug 1"
fi

# if the (possibly new) 1st arg is numeric, it is the port number
if [ $# -eq 0 ] ; then
  :
elif expr \( $1 : '[0-9]*' \) '=' \( $1 : '.*' \) 1> /dev/null; then
  OSAGENT_PORT=$1 export OSAGENT_PORT
  shift
fi

# if arguments remain, we have an error
if [ $# -gt 0 ] ; then
  echo num args $# arg1 is $1
  echo usage:  vbstart [-q] [port-number]
  echo 	where -q turns off server verbosity
  echo  and port-number is the integer value for OSAGENT_PORT
  exit 1
fi

# check that OSAGENT_PORT is set
echo osagent using port ${OSAGENT_PORT:?"OSAGENT_PORT must be set."}.  \
Logs are created in ${logpath}.

# terminate any prior processes
(set +e; GEE_DESTINATION_DIRECTORY/bin/vbstop) 1>/dev/null 2>&1

# create the directory for logs
(set +e; /bin/mkdir ${logpath}) 1>/dev/null 2>&1

# set environment variables, ala Java
CLASSPATH=GEE_JAVA_BASECLASSPATH:GEE_JAVA_CLASSPATH export CLASSPATH
JAVA_HOME=GEE_JAVA export JAVA_HOME
ALT_JDK=GEE_JAVA11 export ALT_JDK

#set environment variables, ala VisiBroker
VBROKER_ADM=${vbroker}/adm  export VBROKER_ADM
LD_LIBRARY_PATH=${vbroker}/lib:${LD_LIBRARY_PATH} export LD_LIBRARY_PATH


PATH=${JAVA_HOME}/bin:${vbroker}/bin:${PATH} export PATH

#start servers
${vbroker}/bin/osagent ${verbflag} 1>> ${logpath}/osagent.log 2>&1 &
echo $! >> ${logpath}/pids
/bin/sleep 5
${vbroker}/bin/oad ${verbflag}     1>> ${logpath}/oad.log 2>&1 &
echo $! >> ${logpath}/pids
/bin/sleep 5
${vbroker}/bin/locserv ${verbloc}  1>> ${logpath}/locserv.log 2>&1 &
echo $! >> ${logpath}/pids

#${vbroker}/bin/gatekeeper 	   1>> ${logpath}/gatekeeper.log 2>&1 &
#echo $! >> ${logpath}/pids

# eight hours from now, kill off these servers
(sleep 28800; GEE_DESTINATION_DIRECTORY/bin/vbstop) &
killpid=$!
echo ${killpid} >> ${logpath}/pids
sleepid=`ps -ef | awk '$3=='${killpid}'{print $2}'`
echo ${sleepid} >> ${logpath}/pids
