package edu.cmu.sei.gee;

import edu.cmu.sei.gee.Log;


/** Provides for debugging.  Debug information is logged to a log dedicated to
	the purpose.
<p>
	All debugging should be conditional based on Debug.isIn:
		if (Debug.isIn) { debugging code }
	Since isIn is static final, the code will be excluded if
	compiled when isIn is false.
<p>
	Four levels of debug are defined, numbered 0 for none
	thru 3 for buy-another-disk mode.  The level can be set
	dynamically or from gee.properties.

*/
public class Debug {

/** Indicates whether debugging code should be compiled in.
*/
public static final boolean isIn = true;


/** Value for `level'.  Indicates that no debugging information is logged. */
public final static int NONE = 0;

/** Value for `level'.  Indicates that a little debug output is desired. */
public final static int TOOLITTLE = 1;

/** Value for `level'.  Indicates that a moderate amount of info is wanted */
public final static int ENOUGH = 2;

/** Value for `level'.  Indicates a glutton for reading. */
public final static int TOOMUCH = 3;


/* Indicates current amount of debug output.  Uses the four defined values:
	NONE, TOOLITTLE, ENOUGH, TOOMUCH.  These are in numerical sequence 
	so the usual code sequence would be something like
<pre>
	if (Debug.isIn && Debug.level >= Debug.ENOUGH) {
		log something
	}
</pre>
*/
public static int level = NONE;


/** The log used for debugging.  Programs can enter() msgs directly,
	but are encouraged to use Debug.show() instead. 
*/
static public Log debugLog = new Log(null, "debug");


/** Makes an entry in the debug log. */
static public void show(String msg) {
	debugLog.enter(msg);
}


/** Makes an entry in the debug log and includes a partial backtrace.
	@param depth The number of levels of backtrace to include. 
*/
public static synchronized void backtrace(int depth, String msg) {
	debugLog.enter(msg);
	// xxx -- make a backtrace and log the `depth' latest levels
}


// xxx -- ?? should we have multiple debug logs
// xxx -- ?? should it be possible to associate a debug log with a 
// xxx --	particular class or object


} // end class Debug
