package edu.cmu.sei.gee;

import edu.cmu.sei.gee.Log;


/** Provides for handling and reporting errors.
<p>
	Gee error policy.
<p>
	In general an error should generate an exception so a calling routine
	can deal with the problem.  When generating an exception, no message
	should be printed.
<p>
	Exceptions should be passed upward in their original form until a
	level where they can be aggrregated into a Gee failure.  At that time
	they can be replaced with a subclass of GeeException. 

@see GeeException

*/
public class Error {

private static String buildMsg(String msg, Object obj, Exception ex) {
	String val = msg;
	if (obj != null) 
		val += "  for object " + obj.toString();
	if (ex != null)
		val += "  for exception " + ex.toString();
	return val;
}


/** Handles a fatal error and terminates program execution.  
	Error information is logged and printed.
@returns This method DOES NOT RETURN.
*/
public static void fatal (String msg, Object obj, Exception ex) {
	String fullMsg = "Terminating execution. " + buildMsg(msg, obj, ex);
	Log.std.enter(fullMsg);
	System.err.println(fullMsg);
	System.exit(91);
}


/** Handles a non-fatal error.
	Error information is logged and printed.
*/
public static void note (String msg, Object obj, Exception ex) {
	String fullMsg = "*** Error: " + buildMsg(msg, obj, ex);
	Log.std.enter(fullMsg);
	System.err.println(fullMsg);
}


} // end class Error
