package edu.cmu.sei.gee;

import java.io.Serializable;


/** Provides a canonical form for a person's name.
<p>
	Names can be formatted for display or printing with an
	object which is a subclass of NameFormat, as perhaps 
	SimpleNameFormat.
*/
public class Name implements Serializable, Cloneable, Comparable {


private String familyName, givenNames, prefix = "", postfix = "";


/** Creates a Name object for a surName and properNames.
	The postfix and prefix and empty.
*/
public Name(String familyName, String givenNames) {
	this(familyName, givenNames, "", "");
}

/** Creates a Name object for a surName and properNames, 
	with particular postfix and prefix.
*/
public Name(String familyName, String givenNames, 
		String prefix, String postfix) {
	this.familyName = familyName;
	this.givenNames = givenNames;
	this.prefix = prefix;
	this.postfix = postfix;
}

// implements the interface comparable
/** Compares this Name to another. 
@throws ClassCastException
*/
public int compareTo(Object other) throws ClassCastException {
	return this.compareTo((Name)other);
}

/** Compares this Name to another. 
*/
public int compareTo(Name o) {
	return (familyName+givenNames+postfix+prefix)
		.compareTo(o.familyName+o.givenNames+o.postfix+o.prefix);
}

/** Tests whether this Name is the same as another.
@throws ClassCastException
*/
public boolean equals(Object other) throws ClassCastException {
	return this.equals((Name) other);
}

/** Tests whether this Name is the same as another.
*/
public boolean equals(Name o) {
	return (familyName+givenNames+postfix+prefix)
		.equals(o.familyName+o.givenNames+o.postfix+o.prefix);
}

/** Returns a hash code for comparison with other Name values.
	The prefix and postfix are not considered.
*/
public int hashCode() {
	return familyName.hashCode() ^ givenNames.hashCode();
}


/** Returns a String form of the name.  
	The format is familyName comma space givenNames 
		comma space postfix comma space prefix
*/
public String toString() {
	return familyName + "," + givenNames 
	+ ", " + postfix + ", " + postfix;
}

// this identifies this version of the code, 
// so objects are compatible across recompilation
// xxx -- ?? how is the number computed?
public static final long serialVersionUID = 123123123; // ?


} // end class Name
