#!/bin/csh -f

# rcscotree dirpath/dirname
#		(combines functionality of rcslink and rcsupdate)
#	checks . for dirname
#		if it exists, fail
#	create dirname and rcslink it to dirpath/dirname
#	check out the entire tree under dirpath/dirname
#	if there is a subdirs file, use it as list of subdirectories

set RCSROOT = XRCSrootX
set REALRCSLINK = XrcslinkX
set REALRCSCO = XrcscoX
set TEST = XtestX

# establish $arg based on $1 and split to dirname, dirpath
if ($#argv < 1) then
    set arg = $RCSROOT
else if ("$1" !~ /*) then
    set arg = $RCSROOT/$1
else
    set arg = $1
endif
set dirname=${arg:t}
set dirpath=${arg:h}

# check for ./dirname,  fail if it exists
# exists if -e is true or it is a link
if (-e ${dirname} || (`${TEST} -h ${dirname}; echo ${status}` == 0)) then
	echo "***" rcscotree MUST create the destination.
	echo "***    " ${dirname} exists, so nothing is done now
	exit (-1)
endif

if (${dirpath} == ${dirname}) then
	set dirpath=
else
	set dirpath=${dirpath}/
endif

# create dirname, rcslink it, and check out rcs files
/bin/mkdir ./${dirname}
cd ./${dirname}
$REALRCSLINK ${dirpath}${dirname}
$REALRCSCO RCSNAME/*,v

# for each subdirectory of RCS, call rcscotree recursively
# copy each plain file (non-,v) in RCS to .
if (-r subdirs) then
	set sublist=`/bin/cat subdirs`
else
	set sublist=`/bin/ls ./RCSNAME | /bin/egrep -v '.*,v'`
endif
foreach f (${sublist})
	if (-d ./RCSNAME/${f}) then
		$0 ${dirpath}${dirname}/${f}
	else if (-f ./RCSNAME/${f}) then
		# '&&' did not work!
		if (-r ./RCSNAME/${f}) then
			/bin/cp ./RCSNAME/${f} .
		endif
	endif
end
exit (0)
