#!/bin/csh -f
# $Id: rcslink.csh,v 1.12 1998/11/25 20:48:49 wjh Exp $

# These first four are set from variables in the Makefile
set RCSROOT = XRCSrootX
set REALRCSMKDIR = XrcsmkdirX
set REALRCSLINK = XrcslinkX
set TEST = XtestX
set RcsName=RCSNAME

set Cflag = ""
set Fflag = ""
set Rflag = ""
set AllFlags = ""
set RemoteDir = ""

#parse arguments

foreach arg ($*)
    switch ($arg)
	case -r:
	    set Rflag = x
	    set AllFlags = ( $AllFlags -r )
	    breaksw
	case -c:
	    set Cflag = x
	    set AllFlags = ( $AllFlags -c )
	    breaksw
	case -f:
	    set Fflag = x
	    set AllFlags = ( $AllFlags -f )
	    breaksw
	case -*:
	    goto usage
	case /*:
	    if ("$RemoteDir" != "") goto usage
	    set RemoteDir="$arg"
	    breaksw
	case *:
	    if ("$RemoteDir" != "") goto usage
	    set RemoteDir="$RCSROOT/$arg"
	    breaksw
    endsw
end
goto process

usage:
	echo "usage: rcslink [-r] [-c] [-f] [RCSdirectory]"
	echo "	-r = descend tree recursively"
	echo "	-c = create"
	echo "	-f = force (even if there is a link already)"
	exit 1

process:

# analyze 'RCS'  (or whatever is $RcsName)
#	sets:  isnotlink, RcsPath, type
${TEST} -h $RcsName
set isnotlink=$status
if (! $isnotlink) set RcsPath=`/bin/ls -ld $RcsName | /bin/sed -e "s/^.*-> //"`
if (-f $RcsName) then
	set type="file"
else if (-d $RcsName) then
	set type="directory"
else
	set type="absent"
endif	

# check for no argument case (just report the lin)
if ("$RemoteDir" == "") goto describelink

# check for error cases
if ("$type" != "absent" && ($isnotlink || "$Fflag" != "x")) goto describelink


# if $RcsName is a link, delete the old value
#	this happens for -f or a link to nowhere
if (! $isnotlink) then
	echo Deleting existing link to $RcsPath
	/bin/rm -f $RcsName
	${TEST} -h $RcsName
	if ($status == 0) then
		echo ERROR: could not delete $cwd/$RcsName
		exit 4
	endif
endif

# create rcs tree directory if requested
if ( "$Cflag" == "x" ) then
    echo NEW directory in RCS tree:
    echo -n "    "
    $REALRCSMKDIR $RemoteDir
	# HACK: make group writable
    /bin/chmod g+w $RemoteDir
endif

# check existence of rcs tree directory
if ( ! -d $RemoteDir) then
    echo "The RCS directory $RemoteDir does not exist"
    echo "    No link has been created - exiting"
    exit 3
endif


########  Make the link from ./RCS/ ##############
echo "Linking this directory to " $RemoteDir
/bin/ln -s $RemoteDir $RcsName
${TEST} -h $RcsName
if ($status == 1) then
	echo "	ERROR: Link creation failed for $cwd/$RcsName"
	exit 5
endif

# do it recursively if requested
if ( "$Rflag" == "x" ) then
	set lastcwd = $cwd
	set subdirlist=`ls $RcsName/*`
	if ("$subdirlist" == "") exit 0
	foreach subname ( $subdirlist )
	    set subdir = $subname:t 
	    if ( -d $RcsName/$subdir && "$subdir" != "$RcsName" ) then
		if ( ! -d $subdir) /bin/mkdir $subdir
		cd $subdir
		if ($status != 0) then
			echo ERROR: for $cwd/$subdir
			echo "	Either cannot create it or cannot cd to it"
			exit 6
		endif
		echo Running rcslink in $cwd...
		$REALRCSLINK $AllFlags $RemoteDir/$subdir
		if ($status != 0) exit $status
		cd $lastcwd
	    endif
	end
endif

exit 0


describelink:

# report information about 'RCS'
if ($isnotlink) then
	switch($type)
	case file:	
		echo $cwd/$RcsName is a file.
		echo "	ERROR: It should be a directory."
		exit 2
	case directory: 
		if ("$RemoteDir" != "") then
			echo No link made.  $cwd/$RcsName is a local directory 
		else
			echo $cwd/$RcsName
		endif
		exit 0
	case absent: 
		exit 0
	endsw
else
	# is a link
	switch($type)
	case file:	
		echo $cwd/$RcsName is linked to file $RcsPath.  
		echo "	ERROR: It should link to a directory."
		exit 2
	case directory: 
		if ("$RemoteDir" != "") echo -n "ERROR: Already linked to "
		echo $RcsPath
		exit 0
	case absent: 
		echo $cwd/$RcsName is linked to $RcsPath
		echo "	ERROR: The link target does not exist."
		exit 1
	endsw
endif
