/* $Id: rcslocks.c,v 1.7 1998/10/01 01:16:05 wjh Exp $ */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#if (!defined(_IBMR2) && !defined(hpux) && !defined(mips) && !defined(__svr4__))
#include <sys/dir.h>
#define dirent direct
#else
#include <dirent.h>
#endif
#ifndef _S_IFDIR
#define _S_IFDIR S_IFDIR
#endif
#include <string.h>

#define bool short
#define TRUE 1
#define FALSE 0
#define USRLEN 9

char *getenv();

#define ALL 1
#define RECURSIVE 2
#define PNAME 4
#define SILENT 8

/* [Forward] Function Declarations */
bool isdir ( );
int main ( );
static void printusage (  );
static void printhelp (  );
int rcs_scandir (  );
int plocks (  );

/*
 * this program looks for RCS files (files with a ,v suffix) and checks
 * for a string 'locks' and reads the userid that should follow it. As of
 * 4/3/91, this works for both the latest and older versions of RCS.
 */


bool isdir(path)
char *path;
{
    struct stat statbuf;

    stat(path, &statbuf);

    if (statbuf.st_mode & _S_IFDIR)
	return TRUE;
    else
	return FALSE;
}

char *user;

int main(argc, argv)
int argc;
char **argv;
{
    int flags = 0, numlocks = 0;

    while (*++argv != NULL && **argv == '-')
	while (*(++*argv) != '\0')
	    switch (**argv) {
		case 'a':
		    flags |= ALL;
		    break;
		case 'r':
		    flags |= RECURSIVE;
		    break;
		case 'p':
		    flags |= PNAME;
		    break;
		case 's':
		    flags |= SILENT;
		    break;
		case 'x':
		    printhelp();
		    exit(-1);
		case 'h':
		    printhelp();
		    exit(-1);
		default:
		    printusage();
		    exit(-1);
	    }

    user = getenv("USER");

    if (*argv == NULL)
	numlocks = rcs_scandir(".", flags);

    while (*argv != NULL)
	if (isdir(*argv))
	    numlocks += rcs_scandir(*argv++, flags);
	else
	    numlocks += plocks(*argv++, flags);

    if (numlocks > 0)
	exit(0);
    else
	exit(1);
}

static void printusage()
{
    fprintf(stderr, "Usage:\trcslocks [-arps] (file | directory)*\n");
    return;
}


static void printhelp()
{
    fprintf(stderr, "rcslocks [-arps] (file | directory)*\n");
    fprintf(stderr, "List files you have locked in RCS\n");
    fprintf(stderr, "-a\tDisplay locks by all users, not just you\n");
    fprintf(stderr, "-r\tRecursively descend into subdirectories\n");
    fprintf(stderr, "-p\tPrint the name of the locker as well as the filename\n");
    fprintf(stderr, "-s\tSilent mode (return success, 0, or failure, 1)\n");
    return;
}

int rcs_scandir(path, flags)
char *path;
int flags;
{
    char buf[200], *end;
    int numlocks = 0;
    DIR *dir;

    if (strcmp(path, ".") == 0) {
	strcpy(buf, "RCS");
	end = buf;
    }
    else {
	strcpy(buf, path);
	strcat(buf, "/RCS");
	end = buf + strlen(path) + 1;
    }

    dir = opendir(buf);
    if (dir != NULL) {
	struct dirent *dp = readdir(dir);

	while (dp != NULL) {
	    int len = strlen(dp->d_name);
	    if (len >= 2 &&
		dp->d_name[len - 1] == 'v' &&
		dp->d_name[len - 2] == ',') {
		strcpy(end, dp->d_name);
		*(end + len - 2) = '\0'; /* chop ,v */
		numlocks += plocks(buf, flags);
	    }
	    dp = readdir(dir);
	}

	closedir(dir);
    }

    if (flags & RECURSIVE) {
	dir = opendir(path);
	if (dir == NULL)
	    fprintf(stderr, "rcslocks: Can't open directory %s\n", path);
	else {
	    struct dirent *dp = readdir(dir);

	    while (dp != NULL) {
		strcpy(end, dp->d_name);
		if (*dp->d_name != '.' &&
		    isdir(buf) &&
		    strcmp(dp->d_name, "RCS") != 0) {
		    rcs_scandir(buf, flags);
		}
		dp = readdir(dir);
	    }

	    closedir(dir);
	}
    }

    return numlocks;
}

int plocks(path, flags)
char *path;
int flags;
{
    char *match = "\nlocks", *scan = match + 1, *base;
    char locker[200], *p = locker;
    char buf[200];
    int lockercnt;
    int c;
    FILE *fp;

    base = strrchr(path, '/');
    if (base != NULL) {
	strncpy(buf, path, base - path);
	strcpy(buf + (base - path), "/RCS/");
	base++;
    }
    else {
	strcpy(buf, "RCS/");
	base = path;
    }

    strcat(buf, base);
    strcat(buf, ",v");

    fp = fopen(buf, "r");
    if (fp == NULL) {
	fprintf(stderr, "rcslocks: Can't open %s\n", buf);
	return 0;
    }

    while ((c = fgetc(fp)) != EOF) {
	if (c != *scan)
	    scan = match;
	else if (*++scan == '\0')
	    break;
    }

    if (c == EOF) {
	fprintf(stderr, "rcslocks: Can't find a locks line in %s; is it an RCS file?\n", buf);
	fclose(fp);
	return 0;
    }


    while ((c = fgetc(fp)) != EOF)	/* need to skip over white space */
	if ((c != '\n') && (c != '\t') && (c != ' '))
	    break;


    if (c == EOF || c == ';') {
	fclose(fp);
	return 0;
    }

    /*
     * next we read for a userid between "locks" and a ":" into a char
     * array 'locker'. But the userid should not be longer than USRLEN or
     * something is wrong.
     */

    lockercnt = 0;
    do {
	*p++ = c;
	lockercnt++;
    }
    while ((c = fgetc(fp)) != EOF && (c != ':') && (lockercnt != USRLEN));
    *p = '\0';

    fclose(fp);

    if (!(flags & ALL) && strcmp(locker, user) != 0)
	return 0;

    if (!(flags & SILENT)) {
	fputs(path, stdout);
	if (flags & PNAME)
	    printf(": %s", locker);
	putchar('\n');
    }

    return 1;
}
